from .validator import Validator
from .string import String
from .object import Object
from .number import Number
from .array import Array
from .email import Email
from .uuid import UUID

name = "Kanpai"
